require 'active_record'

require 'support/app/question'
require 'support/app/answer'
require 'support/app/parent_and_child_searchable'
require 'support/app/article_with_custom_serialization'
require 'support/app/import_article'
require 'support/app/namespaced_book'
require 'support/app/article_for_pagination'
require 'support/app/article_with_dynamic_index_name'
require 'support/app/episode'
require 'support/app/image'
require 'support/app/series'
require 'support/app/mongoid_article'
require 'support/app/article'
require 'support/app/searchable'
require 'support/app/category'
require 'support/app/author'
require 'support/app/authorship'
require 'support/app/comment'
require 'support/app/post'
